package com.usky.dxtop.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.usky.dxtop.common.core.domain.AjaxResult;
import com.usky.dxtop.common.core.page.CommonPage;
import com.usky.dxtop.common.exception.CustomException;
import com.usky.dxtop.common.utils.Arith;
import com.usky.dxtop.common.utils.QRCodeUtil;
import com.usky.dxtop.common.utils.StringUtils;
import com.usky.dxtop.common.utils.http.HttpUtils;
import com.usky.dxtop.common.utils.sign.Base64;
import com.usky.dxtop.common.utils.spring.GlobalUtils;
import com.usky.dxtop.mapper.OrderMapper;
import com.usky.dxtop.model.CallApiLog;
import com.usky.dxtop.model.Order;
import com.usky.dxtop.service.CallApiLogService;
import com.usky.dxtop.service.OrderService;
import com.usky.dxtop.service.api.TopApi;
import com.usky.dxtop.service.emun.OrderStatus;
import com.usky.dxtop.service.vo.OrderRequest;

import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.record.SSTRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.FastByteArrayOutputStream;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.TreeMap;

/**
 * <p>
 * 订单表 服务实现类
 * </p>
 *
 * @author yq
 * @since 2021-08-26
 */
@Slf4j
@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements OrderService {

    public static final String SCAN_PAY = "扫码支付";

    public static final String FIND_ORDER_INFO = "交易查询";

    public static final String CARD_TOP = "卡充值";

    @Autowired
    private CallApiLogService callApiLogService;


    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean add(Order order) {
        String orderNumber = TopApi.getOrderNumber();
        order.setOrderNumber(orderNumber);
        order.setOrderFlag(OrderStatus.NO_PAYMENT.getCode());
        this.save(order);
        callCardTopApi(order);
        return true;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(String orderNumber) {
        LambdaQueryWrapper<Order> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Order::getOrderNumber,orderNumber);
        this.remove(queryWrapper);
        LambdaQueryWrapper<CallApiLog> callApiQueryWrapper = Wrappers.lambdaQuery();
        callApiQueryWrapper.eq(CallApiLog::getOrderNumber,orderNumber);
        return callApiLogService.remove(callApiQueryWrapper);
    }

    @Override
    public Order one(String orderNumber){
        LambdaQueryWrapper<Order> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Order::getOrderNumber,orderNumber);
        return this.getOne(queryWrapper);
    }

    @Override
    public CommonPage<Order> list(OrderRequest orderRequest) {
        IPage<Order> page = new Page<>(orderRequest.getCurrent(), orderRequest.getSize());
        LambdaQueryWrapper<Order> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .like(StringUtils.isNotBlank(orderRequest.getUserName()),Order::getUserName,orderRequest.getUserName())
                .like(StringUtils.isNotBlank(orderRequest.getUserPhone()),Order::getUserPhone,orderRequest.getUserPhone())
                .eq(null != orderRequest.getPayType(),Order::getPayType,orderRequest.getPayType())
                .between(null != orderRequest.getStartMoney() && null != orderRequest.getEndMoney(),
                        Order::getMoney,orderRequest.getStartMoney(),orderRequest.getEndMoney())
                .between(null != orderRequest.getStartTime() && null != orderRequest.getEndTime(),
                        Order::getCreateTime,orderRequest.getStartMoney(),orderRequest.getEndTime())
                .eq(null != orderRequest.getOrderFlag(),Order::getOrderFlag,orderRequest.getOrderFlag());
        page = this.page(page, queryWrapper);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getCurrent(),page.getSize());
    }


    /**
     * 东信扫码支付api
     * @param order
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String getTopScanPayApi(Order order){
        String orderNumber = TopApi.getOrderNumber();
        order.setOrderNumber(orderNumber);
        TreeMap<String, String> params = TopApi.generateScanPayApiParam(order.getOrderNumber(),
                                        new Double(Arith.mul(order.getMoney(),100)).intValue(), null,
                null,null,null,null,null,null);
        String scanPayUrl = String.format("%s?%s&sign=%s", TopApi.SCAN_TO_PAY_URL,
                TopApi.joinParam(params),
                TopApi.signData(params));
        baseMapper.insert(order);
        //记录调用日志
        callApiLogService.add(orderNumber,SCAN_PAY,TopApi.SCAN_TO_PAY_URL,JSONObject.toJSONString(params),null);
        return scanPayUrl;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public String getTopScanPayQr(Order order) {
        String destPath;
        String encode;
        try {
            //获取url
            String topScanPayApi = getTopScanPayApi(order);
            String codeName = String.format("%s-%s.jpg", "usky", System.currentTimeMillis() + "");
            destPath = String.format("%s%s", GlobalUtils.getTempBaseDir(),codeName);
            //生成二维码
            QRCodeUtil.encode(topScanPayApi, null, destPath, true);

            BufferedImage image = QRCodeUtil.encode(topScanPayApi, null, true);
            // 转换流信息写出
            FastByteArrayOutputStream os = new FastByteArrayOutputStream();
            try
            {
                ImageIO.write(image, "jpg", os);
            }
            catch (IOException e)
            {
                throw new CustomException("接口异常"+e.getMessage());
            }
            encode = Base64.encode(os.toByteArray());
        }catch (Exception e){
            throw new CustomException("生成二维码失败");
        }
        return encode;
    }

    /**
     * 查询支付订单api
     * @param orderNumber
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String callTopFindOrderInfoApi(String orderNumber) {
        TreeMap<String, String> param = TopApi.generateDealSelectParam(orderNumber, null, null, null);
        String format = String.format("%s&sign=%s", TopApi.joinParam(param),
                TopApi.signData(param));
        String result = HttpUtils.sendGet(TopApi.DEAL_SELECT_URL, format);
        callApiLogService.add(orderNumber,FIND_ORDER_INFO,TopApi.DEAL_SELECT_URL,JSONObject.toJSONString(param),result);
        Order order = one(orderNumber);
        //处理订单
        checkOrder(order, result);
        //再此获取订单的最新状态
        order = one(orderNumber);
        return OrderStatus.parse(order.getOrderFlag()).getName();
    }

    @Override
    public void callCardTopApi(Order order) {
        //TODO 等待测试接口,url,请求，响应需要替换
        boolean b = false;
        try {
            String param = "";
            String result = "";
            b = true;
            callApiLogService.add(order.getOrderNumber(),CARD_TOP,TopApi.DEAL_SELECT_URL,JSONObject.toJSONString(param),result);
            if (b){
                //成功修改状态成功
                order.setOrderFlag(OrderStatus.SUCCESS.getCode());
                this.updateById(order);
            }else {
                //失败修改状态为卡充值失败
                order.setOrderFlag(OrderStatus.PAYMENT_ERROR_DEBIT.getCode());
                this.updateById(order);
            }
        }catch (Exception e){
            log.error("---order---卡充值api调用异常"+ e.getMessage());
            throw new CustomException("卡充值异常"+e.getMessage());
        }
    }

    /**
     * 校验是否继续处理订单
     * @param orderNumber
     * @return
     */
    public boolean isCheckOrder(String orderNumber){
        Order order = one(orderNumber);
        //订单是未支付和支付中需要处理
        return OrderStatus.NO_PAYMENT.getCode().equals(order.getOrderFlag()) || OrderStatus.AWAIT_PAY.getCode().equals(order.getOrderFlag());
    }


    /**
     * 查询交易后处理订单
     * @param order
     */
    @Override
    public void checkOrder(Order order,String result){
        JSONObject obj= JSON.parseObject(result);
        if ("成功".equals(obj.get("status"))){
            paySuccess(order);
        }else if ("等待中".equals(obj.get("status"))){
            payWaiting(order);
        }else if ("交易订单不存在".equals(obj.get("status"))){
            dealNo(order);
        }else {
            payError(order);
        }
    }

    /**
     * 充值成功后处理
     * @param order
     */
    @Override
    public void paySuccess(Order order){
        try {
            //是否可以操作
            boolean b = isCheckOrder(order.getOrderNumber());
            if (b){
                //调用卡充值api
                callCardTopApi(order);
            }
        }catch (Exception e){
            log.error("---order---卡充值异常"+e.getMessage());
        }
    }

    /**
     * 充值失败后处理
     * @param order
     */
    public void payError(Order order){
        order.setOrderFlag(OrderStatus.PAYMENT_ERROR_NO_DEBIT.getCode());
        this.updateById(order);
    }

    /**
     * 处理等待中的订单
     * @param order
     */
    public void payWaiting(Order order){
        order.setOrderFlag(OrderStatus.AWAIT_PAY.getCode());
        this.updateById(order);
    }

    /**
     * 交易订单不存在
     * @param order
     */
    public void dealNo(Order order){

    }
}
